"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApiClients = exports.createApiClient = void 0;
var tslib_1 = require("tslib");
var promises_1 = tslib_1.__importDefault(require("fs/promises"));
var fs_extra_1 = require("fs-extra");
var path_1 = require("path");
var node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
var prettier_1 = require("farrow-api/dist/prettier");
var generateApiClient_1 = require("farrow-api-server/dist/generateApiClient");
var farrow_api_server_1 = require("farrow-api-server");
var replaceUrl_1 = require("./replaceUrl");
var writeFile = function (filename, content) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, , 2, 4]);
                return [4 /*yield*/, (0, fs_extra_1.ensureDir)((0, path_1.dirname)(filename))];
            case 1:
                _a.sent();
                return [3 /*break*/, 4];
            case 2: return [4 /*yield*/, promises_1.default.writeFile(filename, content)];
            case 3:
                _a.sent();
                return [7 /*endfinally*/];
            case 4: return [2 /*return*/];
        }
    });
}); };
var createApiClient = function (options) {
    var config = tslib_1.__assign(tslib_1.__assign({}, options), { pollingInterval: 3000 });
    var tid = null;
    var prevText = '';
    var getIntrospection = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var url, response, text;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = (0, farrow_api_server_1.getIntrospectionUrl)(config.src);
                    return [4 /*yield*/, (0, node_fetch_1.default)(url)];
                case 1:
                    response = _a.sent();
                    return [4 /*yield*/, response.text()];
                case 2:
                    text = _a.sent();
                    if (text === prevText) {
                        return [2 /*return*/];
                    }
                    prevText = text;
                    try {
                        return [2 /*return*/, JSON.parse(text)];
                    }
                    catch (error) {
                        console.log("Failed to fetch ".concat(config.src, "\nerror: ").concat(error.message, "\nresponse:").concat(text));
                    }
                    return [2 /*return*/];
            }
        });
    }); };
    var sync = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var result, source;
        var _a;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, getIntrospection()];
                case 1:
                    result = _b.sent();
                    if (!result) {
                        return [2 /*return*/];
                    }
                    source = (0, generateApiClient_1.generateApiClient)(result, tslib_1.__assign(tslib_1.__assign({}, config.codegen), { url: (_a = config.alias) !== null && _a !== void 0 ? _a : config.src }));
                    if (config.transform) {
                        source = config.transform(source);
                    }
                    if (config.format) {
                        source = config.format(source);
                    }
                    else {
                        source = (0, prettier_1.format)(source);
                    }
                    if (typeof config.logger === 'function') {
                        config.logger(options);
                    }
                    else if (config.logger !== false) {
                        console.log("synced", {
                            src: config.src,
                            dist: config.dist,
                        });
                    }
                    return [4 /*yield*/, writeFile(config.dist, source)];
                case 2:
                    _b.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    var start = function () {
        stop();
        tid = setInterval(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var error_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, sync()];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        error_1 = _a.sent();
                        console.log(error_1.stack);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); }, config.pollingInterval);
    };
    var stop = function () {
        if (tid !== null) {
            clearInterval(tid);
        }
    };
    var build = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, (0, replaceUrl_1.replaceUrl)(options)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    return {
        sync: sync,
        start: start,
        stop: stop,
        build: build,
    };
};
exports.createApiClient = createApiClient;
var createApiClients = function (options) {
    var clients = options.services.map(exports.createApiClient);
    var sync = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var promises;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    promises = clients.map(function (client) { return client.sync(); });
                    return [4 /*yield*/, Promise.all(promises)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    var start = function () {
        clients.map(function (client) { return client.start(); });
    };
    var stop = function () {
        clients.map(function (syncer) { return syncer.stop(); });
    };
    var build = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var promises;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    promises = clients.map(function (client) { return client.build(); });
                    return [4 /*yield*/, Promise.all(promises)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); };
    return {
        sync: sync,
        start: start,
        stop: stop,
        build: build,
    };
};
exports.createApiClients = createApiClients;
//# sourceMappingURL=index.js.map